package net.educoder.common;


import cn.hutool.core.codec.Base64;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;

/**
 * @Author: youys
 * @Date: 2022/4/2
 * @Description: 钉钉机器人发消息.
 */
@Slf4j
public class DingTalk {

    private String token;
    private String secret;

    public DingTalk(String token, String secret) {
        this.token = token;
        this.secret = secret;
    }

    /**
     * 发送钉钉消息.
     *
     * @param message 消息内容
     * @return string
     */
    public String sendMessage(String message) {
        String url = getUrl();
        JSONObject jsonObject = new JSONObject();
        //固定参数
        jsonObject.put("msgtype", "text");
        JSONObject content = new JSONObject();
        //此处message是你想要发送到钉钉的信息
        content.put("content", message);
        jsonObject.put("text", content);
        try {
            return HttpUtil.post(url, jsonObject.toJSONString());
        } catch (Exception e) {
            log.error("发送钉钉消息异常", e);
            return null;
        }
    }


    private String getUrl() {

        String baseUrl = "https://oapi.dingtalk.com/robot/send?access_token=";
        long timestamp = System.currentTimeMillis();
        String stringToSign = timestamp + "\n" + secret;
        // MAC加密算法
        Mac mac = null;
        try {
            mac = Mac.getInstance("HmacSHA256");
            mac.init(new SecretKeySpec(secret.getBytes(StandardCharsets.UTF_8), "HmacSHA256"));
            byte[] signData = mac.doFinal(stringToSign.getBytes(StandardCharsets.UTF_8));
            return baseUrl + token + "&timestamp=" + timestamp + "&sign=" +
                    URLEncoder.encode(Base64.encode(signData), "UTF-8");
        } catch (Exception e) {
            return null;
        }

    }
}
