package net.educoder.controller;

import cn.hutool.core.codec.Base64;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import net.educoder.common.DingTalk;
import net.educoder.config.PropertiesConfig;
import net.educoder.util.RedisUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @Author: youys
 * @Date: 2022/4/2
 * @Description: 回调接口
 */
@Slf4j
@RestController
@RequestMapping("/callback")
public class CallbackController {

    @Autowired
    private DingTalk dingTalk;

    @Autowired
    private PropertiesConfig propertiesConfig;

    @Autowired
    private RedisUtil redisUtil;

    private final Object LOCK = new Object();


    private static final String RESULT = "success";

    private static List<String> ERROR_MSG_LIST = null;

    @PostMapping("/evaluationResult")
    public String evaluation(@RequestParam Map<String, Object> params) {
        log.info("回调请求参数:{}", params);

        if (params == null || params.get("jsonTestDetails") == null) {
            return RESULT;
        }

        JSONObject jsonTestDetails = JSONObject.parseObject(params.get("jsonTestDetails").toString());
        String outPut = jsonTestDetails.getString("outPut");
        String tpiID = jsonTestDetails.getString("tpiID");

        String decodeOutPut = Base64.decodeStr(outPut);
        log.info("tpiID:{}, decodeOutPut:{}", tpiID, decodeOutPut);

        long incr = 0;
        String incrKey = String.format("evaluationResult:%s", tpiID);
        if (matchOutPut(decodeOutPut) && (incr = redisUtil.increment(incrKey)) > 3) {
            log.info("evaluationResult, tpiID:{},第{}次评测失败,需要发送钉钉通知", tpiID, incr);
            // 钉钉通知
            StringBuilder sb = new StringBuilder();
            sb.append("【自动化评测】tpiID:").append(tpiID).append(" 错误:消息").append(decodeOutPut);
            String result = dingTalk.sendMessage(sb.toString());
            log.info("发送钉钉通知结果:{}", result);

            // 通知之后清空
            redisUtil.remove(incrKey);
            return RESULT;
        } else if (incr == 0L) {
            log.info("tpiID:{},评测通过", tpiID);
        }

        return RESULT;
    }

    /**
     * 匹配输出包含了错误.
     *
     * @param output
     * @return
     */
    private boolean matchOutPut(String output) {
        for (String s : getErrorOutputList()) {
            if (output.contains(s)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 获取错误输出配置列表.
     *
     * @return list
     */
    private List<String> getErrorOutputList() {
        if (ERROR_MSG_LIST == null) {
            synchronized (LOCK) {
                if (ERROR_MSG_LIST == null) {
                    ERROR_MSG_LIST = Arrays.asList(propertiesConfig.getErrorOutPuts().split("\n"));
                }
            }
        }
        return ERROR_MSG_LIST;
    }

}
